#!/usr/bin/env python
# -*- Python -*-

import sys
import time
# import os
import copy
import subprocess

#
# set up user environment
#  RtmToolsDir, MyRtcDir, etc.
#
from set_env import *
from armpos_data import *
#
# import user tools
#
save_path = sys.path[:]
sys.path.append(Rh2LibDir)
from rtc_handle import *
sys.path.append(Rh2LibDir+'/tools/geo')
from geo import *

#
# import stub files
# 
sys.path.append(Rh2IdlDir)
import _GlobalIDL
sys.path = save_path

def assign_variables(env) :
  # rtc
  global RH2ArmControl, RHdisp, RMRC, HandWrap
  RMRC = env.name_space[Rh2ArmNS].rtc_handles['RMRC0.rtc']
  RH2ArmControl = env.name_space[Rh2ArmNS].rtc_handles['RH2ArmControl0.rtc']
  HandWrap = env.name_space[Rh2ArmNS].rtc_handles['RH_HandWrapper0.rtc']
  HandWrap.services['ComRHHandWrap'].provided['com_tool_ctrl'].narrow_ref(globals())
  RMRC.services['ComRMRC'].provided['com_frm_ctrl'].narrow_ref(globals())
  RMRC.services['ComRMRC'].provided['com_fk'].narrow_ref(globals())
  RMRC.services['ComRMRC'].provided['com_move'].narrow_ref(globals())
  RMRC.conf_ref.activate_configuration_set('RefHardArm')
  # data & class
  global c_frame, m_tool_frm, ref_xyz, ref_abc
  c_frame=_GlobalIDL.Frame
  c_vtr = _GlobalIDL.Vector
  ref_xyz= c_vtr(VECTOR())
  ref_abc= c_vtr(VECTOR())
  m_tool_frm = c_frame(MATRIX(),VECTOR())

def make_connectors(env):
  env.con_list=[]
#0
  con = IOConnector([RH2ArmControl.outports['OutArmAngle'],HandWrap.inports['ThPos']])
  env.con_list.append(con)
#1
  con = IOConnector([HandWrap.outports['Th'],RMRC.inports['Th']])
  env.con_list.append(con)
#2
  con = IOConnector([HandWrap.outports['JointGripVel'],RH2ArmControl.inports['InArmVelocity']])
  env.con_list.append(con)
#3
  con = IOConnector([RMRC.outports['Vel'],HandWrap.inports['JointVel']])
  env.con_list.append(con)



#
# connection control
#
def disconnect_all(env) :
  for con in env.con_list :
    con.disconnect()
def connect_list(env,ll) :
  for i in ll :
    env.con_list[i].connect()
def disconnect_list(env,ll) :
  for i in ll :
    env.con_list[i].disconnect()
#
# mode control
##
def mode_joint(env) :
  deactivate_all()
  time.sleep(3)
  disconnect_all(env)
  RH2ArmControl.inports['InArmVelocity'].open()
  RH2ArmControl.outports['OutArmAngle'].open()
  activate_seq([RH2ArmControl])
#
def mode_rmrc(env) :
  deactivate_all()
  disconnect_all(env)
  RH2ArmControl.inports['InArmVelocity'].close()
  RH2ArmControl.outports['OutArmAngle'].close()
  time.sleep(3)
  connect_list(env,[0,1,2,3])
  activate_seq([RH2ArmControl, RMRC, 3])
  tmp=FRAME(xyzabc=[0,0,130.5,0,pi,0])
  m_tool_frm.mat = tmp.mat
  m_tool_frm.pos = tmp.vec
  RMRC.services['ComRMRC'].provided['com_fk'].ref.set_tool(m_tool_frm)
  RMRC.services['ComRMRC'].provided['com_frm_ctrl'].ref.set_param(1.0, 100, 200)
  activate_seq([HandWrap])
#
def arm_release(env) :
  deactivate_all()
  disconnect_all(env)
  RH2ArmControl.inports['InArmVelocity'].close()
  RH2ArmControl.outports['OutArmAngle'].close()
#
def activate_seq(a_seq) :
  for tmp in a_seq :
    if isinstance(tmp,RtcHandle) :
      tmp.activate()
    elif isinstance(tmp, int) :
      time.sleep(tmp)
    elif isinstance(tmp, float) :
      time.sleep(tmp)
    else :
      eval(tmp)
#
def deactivate_all() :
  RH2ArmControl.deactivate()
  RMRC.deactivate()
  HandWrap.deactivate()

# some joint configurations
#
j_park = [0.0,0.0,0.0,0.0,0.0,0.0,0.0]
j_ready = [0.0,0.5,1.6,0.0,1.04,0.0,0.0]
j_squat = [0.0,-1.13,2.6,0.0,0.9,0.0,0.0]
j_escape = [0.0,2.0,0.0,0.0,0.0,0.0,0.0]
#
# joint motion
#
def move_joint(Arm_ctrl, angles, k_pos=1.0, v_max=0.4, lim=0.001) :
  v_min = [0.006, 0.005, 0.006, 0.012, 0.014, 0.031, 0.001]  #minimum velocity
  while 1:
    cur=Arm_ctrl.outports['OutArmAngle'].read().data
    if len(cur) != 7 :
      print cur
    else :
      diff=map(lambda x,y:x-y, cur, angles)
      dm=max(map(abs,diff))
      vel=map(lambda x: - k_pos * x, diff)
      for i in range(7) :
        if abs(diff[i]) < lim :
          vel[i] = 0.0000
      if vel == [0.000,0.000,0.000,0.000,0.000,0.000,0.000] :
          Arm_ctrl.inports['InArmVelocity'].write(vel)
          break
      vm = max(map(abs,vel))
      if vm > v_max :
        vel=map(lambda x : x * v_max/vm, vel)
      for i in range(7) :
        if abs(vel[i]) < v_min[i] and abs(vel[i]) > 0 :
          vel[i] = vel[i] / abs(vel[i]) * v_min[i]
      Arm_ctrl.inports['InArmVelocity'].write(vel)
    time.sleep(0.04)
  return cur
#
def demo(mv_rtc, pos_list) :
     for pos in pos_list :
       go_to(mv_rtc, pos)
#
def go_to(mv_rtc, pos) :
   for i in range(3):
      ref_xyz.pos[i] = pos[i]
      ref_abc.pos[i] = pos[i+3]
   mv_rtc.services['ComRMRC'].provided['com_move'].ref.move_to(ref_xyz, ref_abc)
#
def demo2(ctrl_rtc, pos_list) :
     for pos in pos_list :
       go_to2(ctrl_rtc, pos)
#
# set RMRC mode or restart RHarm
def ready(env, flg=0) :
   if(flg == 0) :
      mode_joint(env)
      time.sleep(3)
      move_joint(RH2ArmControl,j_park)
      print "joint angles cleared. ready to RESTART!"
   elif(flg == 1) :
      mode_joint(env)
      time.sleep(3)
      move_joint(RH2ArmControl,j_ready)
      mode_rmrc(env)
      print "ready to RMRC!"
#
def gripper_open() :
   HandWrap.services['ComRHHandWrap'].provided['com_tool_ctrl'].ref.gripper_open()
#
def gripper_close() :
   HandWrap.services['ComRHHandWrap'].provided['com_tool_ctrl'].ref.gripper_close()
#
def gripper_stop() :
   HandWrap.services['ComRHHandWrap'].provided['com_tool_ctrl'].ref.gripper_stop()
#
def get_cond() :
   cond = HandWrap.services['ComRHHandWrap'].provided['com_tool_ctrl'].ref.get_cond()
   if (cond == 0) :
     print "Close completed!"
   elif (cond == 1) :
     print "Open completed!"
   elif (cond == 2) :
     print "Now moving!"
   elif (cond == 3) :
     print "Interrupted!"
#
def get_mode() :
   mode = HandWrap.services['ComRHHandWrap'].provided['com_tool_ctrl'].ref.get_mode()
   if (mode == 0) :
     print "Mode : READY"
   elif (mode == 1) :
     print "Mode : OPEN"
   elif (mode == 2) :
     print "Mode : CLOSE"
#
# pick up a can on my drink holder
def holder_pickup(menu, flg=0) :
   global RMRC,pc,pd,pe,drink
   #set_tool(0.0,0.0,120.5)
   if (flg == 0) :
      if (get_cond() != 1) :
         gripper_open()
      demo(RMRC,[pc,pd,pe])
   #can1
   if(menu == "can1") :
      go_to(RMRC,offset(drink["can3"],0.0,0.0,84.5))
      go_to(RMRC,drink["can3"])
      go_to(RMRC,drink["can1"])
      go_to(RMRC,offset(drink["can1"],0.0,-7.0,0.0))
      gripper_close()
      time.sleep(4)
      go_to(RMRC,offset(drink["can1"],0.0,-7.0,84.5))
   #can2,3
   else :
      go_to(RMRC,offset(drink[menu],0.0,0.0,84.5))
      go_to(RMRC,drink[menu])
      go_to(RMRC,offset(drink[menu],0.0,-7.0,0.0))
      gripper_close()
      time.sleep(4)
      go_to(RMRC,offset(drink[menu],0.0,-7.0,84.5))
   demo(RMRC,[pe,pd,pc])
#
# return "pos" to which x_offset, y_offset, and z_offset are added
def offset(pos,x_offset,y_offset,z_offset) :
   off = [x_offset,y_offset,z_offset, 0, 0, 0]
   abv_pos = copy.deepcopy(pos)
   abv_pos = map((lambda x,y:x+y),abv_pos,off)
   return abv_pos
#
# take out a trush from the holder
def trush_clean(trush, flg=0) :
   print "pick up a can on my back."
   holder_pickup(trush, flg)
   go_to(RMRC,ptrush)
   print "and carry it to a trush box."
   gripper_open()
   time.sleep(8)
   gripper_close()
   go_to(RMRC,pc)
#
# create env
#
#env = RtmEnv(sys.argv,[Rh2ArmNS])
#for ns in env.name_space :
#  env.name_space[ns].list_obj()
#assign_variables(env)
#make_connectors(env)

