#!/usr/bin/env python
# -*- Python -*-

import sys
import time
# import os
import subprocess

#
# set up user environment
#  RtmToolsDir, MyRtcDir, etc.
#
from set_env import *
#
# import user tools
#
save_path = sys.path[:]
sys.path.append(Rh2LibDir)
from rtc_handle import *
sys.path.append(Rh2LibDir+'/tools/geo')
from geo import *

#
# import stub files
# 
sys.path.append(Rh2IdlDir)
import _GlobalIDL
sys.path = save_path

#
# run components
#
def run_components() :
  com_RMRC="cd "+Rh2RtcDir+"; ./RMRCComp -f "+Rh2ConfDir+"/RMRC.conf > /dev/null &"
  subprocess.Popen(com_RMRC, shell=True)
  com_HandWrap="cd "+Rh2RtcDir+";./RH_HandWrapperComp -f "+Rh2ConfDir+"/RH_HandWrapper.conf > /dev/null &"
  subprocess.Popen(com_HandWrap, shell=True)
  #com_RH2ArmControl="cd "++Rh2RtcDir+" ./RH2ArmControlComp -f "+Rh2ConfDir+"/RH2ArmCtrl.conf > /dev/null &"
  #print(com_RH2ArmControl)
  #subprocess.Popen(com_RH2ArmControl, shell=True)
  #time.sleep(65) 
  
def assign_variables(env) :
  # rtc
  global RH2ArmControl, RHdisp, RMRC, HandWrap
  RMRC = env.name_space[Rh2ArmNS].rtc_handles['RMRC0.rtc']
  RH2ArmControl = env.name_space[Rh2ArmNS].rtc_handles['RH2ArmControl0.rtc']
  HandWrap = env.name_space[Rh2ArmNS].rtc_handles['RH_HandWrapper0.rtc']
  HandWrap.services['ComRHHandWrap'].provided['com_tool_ctrl'].narrow_ref(globals())
  RMRC.services['ComRMRC'].provided['com_frm_ctrl'].narrow_ref(globals())
  RMRC.services['ComRMRC'].provided['com_fk'].narrow_ref(globals())
  RMRC.services['ComRMRC'].provided['com_move'].narrow_ref(globals())
  RMRC.conf_ref.activate_configuration_set('RefHardArm')
  # data & class
  global c_frame, m_tool_frm, ref_xyz, ref_abc
  c_frame=_GlobalIDL.Frame
  c_vtr = _GlobalIDL.Vector
  ref_xyz= c_vtr(VECTOR())
  ref_abc= c_vtr(VECTOR())
  m_tool_frm = c_frame(MATRIX(),VECTOR())

def make_connectors(env):
  env.con_list=[]
#0
  con = IOConnector([RH2ArmControl.outports['OutArmAngle'],HandWrap.inports['ThPos']])
  env.con_list.append(con)
#1
  con = IOConnector([HandWrap.outports['Th'],RMRC.inports['Th']])
  env.con_list.append(con)
#2
  con = IOConnector([HandWrap.outports['JointGripVel'],RH2ArmControl.inports['InArmVelocity']])
  env.con_list.append(con)
#3
  con = IOConnector([RMRC.outports['Vel'],HandWrap.inports['JointVel']])
  env.con_list.append(con)



#
# connection control
#
def disconnect_all(env) :
  for con in env.con_list :
    con.disconnect()
def connect_list(env,ll) :
  for i in ll :
    env.con_list[i].connect()
def disconnect_list(env,ll) :
  for i in ll :
    env.con_list[i].disconnect()
#
# mode control
##
def mode_joint() :
  deactivate_all()
  time.sleep(5)
  disconnect_all(env)
  RH2ArmControl.inports['InArmVelocity'].open()
  RH2ArmControl.outports['OutArmAngle'].open()
  activate_seq([RH2ArmControl])

def mode_rmrc() :
  deactivate_all()
  disconnect_all(env)
  RH2ArmControl.inports['InArmVelocity'].close()
  RH2ArmControl.outports['OutArmAngle'].close()
  time.sleep(5)
  connect_list(env,[0,1,2,3])
  activate_seq([RH2ArmControl, RMRC, 3])
  tmp=FRAME(xyzabc=[0,0,130.5,0,pi,0])
  m_tool_frm.mat = tmp.mat
  m_tool_frm.pos = tmp.vec
  RMRC.services['ComRMRC'].provided['com_fk'].ref.set_tool(m_tool_frm)
  RMRC.services['ComRMRC'].provided['com_frm_ctrl'].ref.set_param(1.0, 100, 200)
  activate_seq([HandWrap])



def activate_seq(a_seq) :
  for tmp in a_seq :
    if isinstance(tmp,RtcHandle) :
      tmp.activate()
    elif isinstance(tmp, int) :
      time.sleep(tmp)
    elif isinstance(tmp, float) :
      time.sleep(tmp)
    else :
      eval(tmp)

def deactivate_all() :
  RH2ArmControl.deactivate()
  RMRC.deactivate()
  HandWrap.deactivate()

#
# create env
#
env = RtmEnv(sys.argv,[Rh2ArmNS])
run_components()
time.sleep(10)
for ns in env.name_space :
  env.name_space[ns].list_obj()
assign_variables(env)
#make_connectors(env)
#connect_list(env,[0,1,2,3])
