#!/usr/bin/env python
# -*- Python -*-

import sys
import time
import os

#
# set up user environment
#  RtmToolsDir, RtmDir, etc.
#
sys.path.append("..")
from set_env import *   

#
# import user tools
#
sys.path.append(".")
save_path = sys.path[:]
sys.path.append(Rh2LibDir)
from rtc_handle import *
sys.path = save_path

#
# import stub files
# 
#import _GlobalIDL

def assign_variables(env) :
  # rtc
  global navigation, pathfollower
  global teststart, testgoal, dijkstra 
  global hazardmesh, managementofareamap, pathcnv
  global joystick, switch, rh2, odometry
  global localization, camera, ceilingnav
  global pos2pos2d, pos2d2pos
  navigation = env.name_space[Rh2BaseNS1].rtc_handles['Navigation0.rtc']
  pathfollower = env.name_space[Rh2BaseNS1].rtc_handles['PathFollower0.rtc']
  teststart =env.name_space[Rh2BaseNS2].rtc_handles['TestInput0START.rtc']
  testgoal = env.name_space[Rh2BaseNS2].rtc_handles['TestInput0GOAL.rtc']
  dijkstra = env.name_space[Rh2BaseNS2].rtc_handles['Dijkstra0.rtc']
  hazardmesh = env.name_space[Rh2BaseNS2].rtc_handles['HazardMesh0.rtc']
  managementofareamap = env.name_space[Rh2BaseNS2].rtc_handles['ManagementOfAreaMap0.rtc']
  pathcnv = env.name_space[Rh2BaseNS2].rtc_handles['PathCnv0.rtc']
  switch = env.name_space[Rh2BaseNS1].rtc_handles['SwitchInputRTC0.rtc']
  rh2 = env.name_space[Rh2BaseNS1].rtc_handles['RefHardRh20.rtc']
  odometry = env.name_space[Rh2BaseNS1].rtc_handles['Odometry0.rtc']
  localization = env.name_space[Rh2BaseNS1].rtc_handles['LocalizeCenter0.rtc']
  camera = env.name_space[Rh2BaseNS1].rtc_handles['CameraEye0.rtc']
  ceilingnav = env.name_space[Rh2BaseNS1].rtc_handles['CeilingNavigation0.rtc']
  #pos2pos2d = env.name_space[Rh2BaseNS1].rtc_handles['TimedPosition2TimedPose2D0.rtc']
  pos2d2pos = env.name_space[Rh2BaseNS1].rtc_handles['TmPose2D_TmPosition0.rtc']



def make_connectors(env):
  env.con_list=[]

  ## [ManagementOfAreaMap] -> [HazardMesh]
  con = IOConnector([managementofareamap.outports['MapData'],hazardmesh.inports['MapData']])
  env.con_list.append(con)
  ## [HazardMesh] -> [ManagementOfAreaMap]
  con = IOConnector([hazardmesh.outports['MapOrder'],managementofareamap.inports['MapOrder']])
  env.con_list.append(con)
  ## [HazardMesh] -> [Dijkstra]
  con = IOConnector([hazardmesh.outports['MeshData'],dijkstra.inports['MeshData']])
  env.con_list.append(con)
  ## [TestInput] -> [Dijkstra]
  con = IOConnector([teststart.outports['Position'],dijkstra.inports['RobotPosition']])
  env.con_list.append(con)
  con = IOConnector([testgoal.outports['Position'],dijkstra.inports['GoalPosition']])
  env.con_list.append(con)
  ## [Dijkstra] -> [HazardMesh]
  con = IOConnector([dijkstra.outports['MeshOrder'],hazardmesh.inports['MeshOrder']])
  env.con_list.append(con)
  ## [Dijkstra] -> [PathCnv]
  con = IOConnector([dijkstra.outports['Path'],pathcnv.inports['Path']])
  env.con_list.append(con)
  ## [PathCnv] -> [Navigation]
  con = IOConnector([pathcnv.outports['min_path'],navigation.inports['min_path']])
  env.con_list.append(con)
  ## [Navigation] -> [PathFollower]
  con = IOConnector([navigation.outports['path'],pathfollower.inports['target_path']])
  env.con_list.append(con)
  con = IOConnector([navigation.outports['max_speed'],pathfollower.inports['max_speed']])
  env.con_list.append(con)
  con = IOConnector([navigation.outports['max_acceleration'],pathfollower.inports['max_acceleration']])
  env.con_list.append(con)
  ## [Localization] -> [Navigation]
  con = IOConnector([localization.outports['dp_out0'],navigation.inports['position']])
  env.con_list.append(con)
  ## [Localization] -> [PathFollower]
  con = IOConnector([localization.outports['dp_out0'],pathfollower.inports['position']])
  env.con_list.append(con)
  ## [PathFollower] -> [SwitchComponent]
  con = IOConnector([pathfollower.outports['velocity'],switch.inports['inVelAuto']])
  env.con_list.append(con)
  ## [Odometry] -> [LocalizeCenter]
  con = IOConnector([odometry.outports['OdometryPosition'],localization.inports['dp_odmIn']])
  env.con_list.append(con)
  ## [RefHard2] -> [Odometry]
  con = IOConnector([rh2.outports['OutWheelAngle'],odometry.inports['CurrentWheelAngle']])
  env.con_list.append(con)
  ## [SwitchComp] -> [RefHard2]
  con = IOConnector([switch.outports['outVel'],rh2.inports['InTargetVelocity']])
  env.con_list.append(con)
  ## [Joystick] -> [SwitchComp]
  #con = IOConnector([joystick.outports['VelocityIIS'],switch.inports['inVelJoy']])
  #env.con_list.append(con)
  ## [CeilingNav] -> [LocalizeCenter]
  #con = IOConnector([ceilingnav.outports['CeilingPosition'],localization.inports['dp_ceilIn']])
  #env.con_list.append(con)
  con = IOConnector([ceilingnav.outports['CeilingPosition'],pos2d2pos.inports['in']])
  env.con_list.append(con)
  con = IOConnector([pos2d2pos.outports['out'],localization.inports['dp_ceilIn']])
  env.con_list.append(con)
  ## [Camera] -> [CeilingNav]
  con = IOConnector([camera.outports['ImageData'],ceilingnav.inports['CameraData']])
  env.con_list.append(con)
  ## [LocalizeCenter] -> [DispPosition]
  #con = IOConnector([localization.outports['dp_out0'],dispposition.inports['position']])
  #env.con_list.append(con)
  ## [LocalizeCenter] -> [Odometry]
  con = IOConnector([localization.outports['dp_out0'],odometry.inports['LocalizedPosition']])
  env.con_list.append(con)
  ## [LocalizeCenter] -> [CeilingNav]
  #con = IOConnector([localization.outports['dp_out0'],ceilingnav.inports['LocalizedPosition']])
  #env.con_list.append(con)
  con = IOConnector([localization.outports['dp_out0'],pos2pos2d.inports['in']])
  env.con_list.append(con)
  con = IOConnector([pos2pos2d.outports['out'],ceilingnav.inports['LocalizedPosition']])
  env.con_list.append(con)

def set_config_param() :
  teststart.conf_ref.activate_configuration_set('start')
  testgoal.conf_ref.activate_configuration_set('entrance')
  #initodometry.conf_ref.activate_configuration_set('start')
  dijkstra.conf_ref.activate_configuration_set('rtc-center')
  navigation.conf_ref.activate_configuration_set('rtc-center')
  localization.conf_ref.activate_configuration_set('start')

#
# connection control
#
def connect_all(env) :
  for con in env.con_list :
    con.connect()
def disconnect_all(env) :
  for con in env.con_list :
    con.disconnect()
def connect_list(env,ll) :
  for i in ll :
    env.con_list[i].connect()
def disconnect_list(env,ll) :
  for i in ll :
    env.con_list[i].disconnect()
#
#
#
def activate_local() :
  camera.activate()
  time.sleep(1)
  ceilingnav.activate()
  pos2d2pos.activate()
  #pos2pos2d.activate()
  #joystick.activate()
  switch.activate()
  rh2.activate()
  odometry.activate()
  localization.activate()
def deactivate_local() :
  camera.deactivate()
  #joystick.deactivate()
  pos2d2pos.deactivate()
  #pos2pos2d.deactivate()
  switch.deactivate()
  ceilingnav.deactivate()
  rh2.deactivate()
  odometry.deactivate()
  localization.deactivate()
def activate_all() :
  camera.activate()
  time.sleep(1)
  ceilingnav.activate()
  pos2d2pos.activate()
  #pos2pos2d.activate()
  #joystick.activate()
  switch.activate()
  rh2.activate()
  odometry.activate()
  localization.activate()
  #dispposition.activate()
  pathfollower.activate()
  navigation.activate()
  pathcnv.activate()
  dijkstra.activate()
  hazardmesh.activate()
  managementofareamap.activate()
  teststart.activate()
  testgoal.activate()
def deactivate_all() :
  camera.deactivate()
  #joystick.deactivate()
  switch.deactivate()
  ceilingnav.deactivate()
  rh2.deactivate()
  odometry.deactivate()
  localization.deactivate()
  #dispposition.deactivate()
  pos2d2pos.deactivate()
  pathfollower.deactivate()
  navigation.deactivate()
  pathcnv.deactivate()
  dijkstra.deactivate()
  hazardmesh.deactivate()
  managementofareamap.deactivate()
  teststart.deactivate()
  testgoal.deactivate()
#
#
def moveto(start, goal):
  pos=['start','entrance','room','drinkserve','drinkserve_back','dumpsite']
  s_flg=0
  g_flg=0
  for i in range(len(pos)):
    if(start == pos[i]):
      s_pos=pos[i]
      print "start position:" + s_pos
      s_flg=1
    if(goal == pos[i]):
      g_pos=pos[i]
      print "start position:" + g_pos
      g_flg=1
  if(s_flg == 1 and g_flg == 1):
      teststart.conf_ref.activate_configuration_set(s_pos)
      testgoal.conf_ref.activate_configuration_set(g_pos)
      pathfollower.activate()
      navigation.activate()
      dijkstra.activate()
      hazardmesh.activate()
      managementofareamap.activate()
      pathcnv.activate()
      teststart.activate()
      testgoal.activate()
  else:
      print "error:start or goal is invalid."

def stop():
  pathcnv.deactivate()
  navigation.deactivate()
  pathfollower.deactivate()
  dijkstra.deactivate()
  hazardmesh.deactivate()
  managementofareamap.deactivate()
  teststart.deactivate()
  testgoal.deactivate()

#
# create env
#
#env = RtmEnv(sys.argv,[Rh2BaseNS1,Rh2BaseNS2])
#time.sleep(10)
#raw_input("are you sure that each rtc needed is running?")
#for ns in env.name_space :
#  env.name_space[ns].list_obj()
#assign_variables(env)
