// -*- C++ -*-
/*!
 * @file  BatteryWatcher.cpp
 * @brief RHのバッテリー監視コンポーネント
 * @date $Date$
 *
 * $Id$
 */

#include "BatteryWatcher.h"

// Module specification
// <rtc-template block="module_spec">
static const char* batterywatcher_spec[] =
  {
    "implementation_id", "BatteryWatcher",
    "type_name",         "BatteryWatcher",
    "description",       "RHのバッテリー監視コンポーネント",
    "version",           "1.0.0",
    "vendor",            "RTC-Center",
    "category",          "Battery",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.ThresholdVoltage", "21.0",

    ""
  };
// </rtc-template>

BatteryWatcher::BatteryWatcher(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_RH_VINOut("RH_VIN", m_RH_VIN),
    m_Voltage3Out("Voltage3", m_Voltage3),
    m_Voltage5Out("Voltage5", m_Voltage5),
    m_Voltage12Out("Voltage12", m_Voltage12),
    m_TemperatureOut("Temperature", m_Temperature),
    m_BatteryAlertPort("BatteryAlert"),
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  registerOutPort("RH_VIN", m_RH_VINOut);
  registerOutPort("Voltage3", m_Voltage3Out);
  registerOutPort("Voltage5", m_Voltage5Out);
  registerOutPort("Voltage12", m_Voltage12Out);
  registerOutPort("Temperature", m_TemperatureOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  m_BatteryAlertPort.registerConsumer("BatteryAlert", "BatteryAlert", m_BatteryAlert);
  
  // Set CORBA Service Ports
  registerPort(m_BatteryAlertPort);
  
  // </rtc-template>

}

BatteryWatcher::~BatteryWatcher()
{
}



RTC::ReturnCode_t BatteryWatcher::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("ThresholdVoltage", m_ThresholdVoltage, "21.0");
  
  // 電源ボードのUSBデバイスを開いて初期化する
  usb_handle = m4Init();
  if(!usb_handle){
  	std::cout << "電源デバイスが見つかりませんでした。" << std::endl;
  	return RTC::RTC_ERROR;
  }
  // コンフィグレーション？の設定を行う？
  // 多分やらない。

  // </rtc-template>
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t BatteryWatcher::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t BatteryWatcher::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t BatteryWatcher::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t BatteryWatcher::onActivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t BatteryWatcher::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t BatteryWatcher::onExecute(RTC::UniqueId ec_id)
{
//  float		voltage, vol3, vol5, vol12;
//  short		temperature;
//  char		diagBuff[24];
  m4DiagData	diagData;

  /*
   * 電源デバイスに問い合わせを行う。
   * とれる値は以下の5つ
   * ・電源ボードへの入力電圧（バッテリー電圧）
   * ・3.3V出力の実際の出力電圧
   * ・5V出力の実際の電圧
   * ・12V出力の実際の電圧
   * ・温度
   */
/*
  if(m4FetchDiag(usb_handle, diagBuff) < 0){
  	std::cout << "電源状態の問い合わせに失敗しました。" << std::endl;
  }
*/

  if(m4GetDiagnostics(usb_handle, &diagData) < 0){
  	std::cout << "電源状態の問い合わせに失敗しました。" << std::endl;
  	return RTC::RTC_OK;
  }
  m_RH_VIN.data = diagData.voltage;
  m_RH_VINOut.write(m_RH_VIN);

  m_Voltage3.data = diagData.vol3;
  m_Voltage3Out.write(m_Voltage3);

  m_Voltage5.data = diagData.vol5;
  m_Voltage5Out.write(m_Voltage5);

  m_Voltage12.data = diagData.vol12;
  m_Voltage12Out.write(m_Voltage12);

  m_Temperature.data = diagData.temperature;
  m_TemperatureOut.write(m_Temperature);

  printf("\t%0.2f\t%0.2f\t%0.2f\t%0.2f\t%ld\n",
  									diagData.voltage,
  									diagData.vol3,
  									diagData.vol5,
  									diagData.vol12,
  									diagData.temperature); 

  // しきい値判定を行い、しきい値に達していたらアラートメソッドを呼び出す
  if(m_ThresholdVoltage >= diagData.voltage){
  	// サービスポートが有効か確認する
  	if(CORBA::is_nil(m_BatteryAlert.getObject()) == false){
	  	// サービスポート呼び出し
		m_BatteryAlert->RH_InVoltageAlert(diagData.voltage);
  	}
  }
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t BatteryWatcher::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t BatteryWatcher::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t BatteryWatcher::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t BatteryWatcher::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t BatteryWatcher::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void BatteryWatcherInit(RTC::Manager* manager)
  {
    RTC::Properties profile(batterywatcher_spec);
    manager->registerFactory(profile,
                             RTC::Create<BatteryWatcher>,
                             RTC::Delete<BatteryWatcher>);
  }
  
};


