// -*- C++ -*-
/*!
 * @file  InitOdometry.cpp * @brief Initialize Odometry * $Date$ 
 *
 * $Id$ 
 */
#include "InitOdometry.h"

// Module specification
// <rtc-template block="module_spec">
static const char* initodometry_spec[] =
  {
    "implementation_id", "InitOdometry",
    "type_name",         "InitOdometry",
    "description",       "Initialize Odometry",
    "version",           "1.0.0",
    "vendor",            "RTC-Center",
    "category",          "Localization",
    "activity_type",     "SPORADIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "2",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.01_x", "1.5",
    "conf.default.02_y", "2.0",
    "conf.default.03_th", "0.0",
    ""
  };
// </rtc-template>

InitOdometry::InitOdometry(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_PositionOut("Position", m_Position)

    // </rtc-template>
{
}

InitOdometry::~InitOdometry()
{
}


RTC::ReturnCode_t InitOdometry::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers

  // Set OutPort buffer
  addOutPort("Position", m_PositionOut);

  // Set service provider to Ports

  // Set service consumers to Ports

  // Set CORBA Service Ports

  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("01_x", m_01_x, "1.5");
  bindParameter("02_y", m_02_y, "2.0");
  bindParameter("03_th", m_03_th, "0.0");

  // </rtc-template>
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t InitOdometry::onFinalize()
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t InitOdometry::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t InitOdometry::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

RTC::ReturnCode_t InitOdometry::onActivated(RTC::UniqueId ec_id)
{
  struct timeval tv;
  gettimeofday(&tv, NULL);
  timestamp = tv.tv_sec + (double)tv.tv_usec * 1e-6;
  sec  = (unsigned long int)timestamp;
  nsec = (unsigned long int)(fmod(timestamp, 1) * 1000000);

  RTC_INFO(("[configuration]"));
  RTC_INFO(("01_x  = %lf", m_01_x ));
  RTC_INFO(("02_y  = %lf", m_02_y ));
  RTC_INFO(("03_th = %lf", m_03_th));
  //printf("\n[configuration]\n");
  //printf("01_x  = %lf\n", m_01_x );
  //printf("02_y  = %lf\n", m_02_y );
  //printf("03_th = %lf\n", m_03_th);
  //fflush(stdout); 

  m_Position.tm.sec  = sec;
  m_Position.tm.nsec = nsec;
  m_Position.x = m_01_x;
  m_Position.y = m_02_y;
  m_Position.theta = m_03_th;
  m_PositionOut.write();

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t InitOdometry::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t InitOdometry::onExecute(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t InitOdometry::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t InitOdometry::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t InitOdometry::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t InitOdometry::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t InitOdometry::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


extern "C"
{
 
  void InitOdometryInit(RTC::Manager* manager)
  {
    coil::Properties profile(initodometry_spec);
    manager->registerFactory(profile,
                             RTC::Create<InitOdometry>,
                             RTC::Delete<InitOdometry>);
  }
  
};



