// -*- C++ -*-
/*!
 * @file  MnpData_Hub.cpp
 * @brief separate joint data from grip data
 * @date $Date$
 *
 * $Id$
 */

#include "MnpData_Hub.h"

// Module specification
// <rtc-template block="module_spec">
static const char* mnpdata_hub_spec[] =
  {
    "implementation_id", "MnpData_Hub",
    "type_name",         "MnpData_Hub",
    "description",       "separate joint data from grip data",
    "version",           "1.0.0",
    "vendor",            "FSI",
    "category",          "Category",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
MnpData_Hub::MnpData_Hub(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_th_posIn("th_pos", m_th_pos),
    m_grip_widOut("grip_wid", m_grip_wid),
    m_thOut("th", m_th)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
MnpData_Hub::~MnpData_Hub()
{
}



RTC::ReturnCode_t MnpData_Hub::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("th_pos", m_th_posIn);
  
  // Set OutPort buffer
  addOutPort("grip_wid", m_grip_widOut);
  addOutPort("th", m_thOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  m_th.data.length(7);
  
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t MnpData_Hub::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t MnpData_Hub::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t MnpData_Hub::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t MnpData_Hub::onActivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t MnpData_Hub::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t MnpData_Hub::onExecute(RTC::UniqueId ec_id)
{
  int i;

  if(m_th_posIn.isNew()) {
    printf("MnpDataHub\n");
    m_th_posIn.read();
    //関節角速度とツール開口幅を分離する
    for(i=0;i<6;i++){
      m_th.data[i] = m_th_pos.data[i];
      printf("m_th[%d] = %f\n",i,m_th.data[i]);
    }
    m_grip_wid.data = m_th_pos.data[6];
    printf("m_grip_wid = %f\n",m_grip_wid.data);
    m_thOut.write();
    m_grip_widOut.write();
  }
  
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t MnpData_Hub::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t MnpData_Hub::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t MnpData_Hub::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t MnpData_Hub::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t MnpData_Hub::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void MnpData_HubInit(RTC::Manager* manager)
  {
    coil::Properties profile(mnpdata_hub_spec);
    manager->registerFactory(profile,
                             RTC::Create<MnpData_Hub>,
                             RTC::Delete<MnpData_Hub>);
  }
  
};


