// -*-C++-*-
/*!
 * @file  com_RMRCSVC_impl.h
 * @brief Service implementation header of com_RMRC.idl
 *
 */


#include "com_RMRCSkel.h"
#include "rtm/idl/BasicDataType.hh"
#include <rtm/SystemLogger.h>

#ifndef COM_RMRCSVC_IMPL_H
#define COM_RMRCSVC_IMPL_H

#include "fk.h"
#include <math.h>
#include <geo_std.h>
 
using namespace RTC;
 
/*!
 * @class ComFkSVC_impl
 * Example class implementing IDL interface ComFk
 */
class ComFkSVC_impl
 : public virtual POA_ComFk,
   public virtual PortableServer::RefCountServantBase
{
 private:
   // Make sure all instances are built on the heap by making the
   // destructor non-public
   //virtual ~ComFkSVC_impl();
   RTC::Logger rtclog;

 public:
  /*!
   * @brief standard constructor
   */
   ComFkSVC_impl();
  /*!
   * @brief destructor
   */
   virtual ~ComFkSVC_impl();

   // attributes and operations
   void set_tool(const Frame& frm);

  FK f_kinema;

};

/*!
 * @class ComFrmCtrlSVC_impl
 * Example class implementing IDL interface ComFrmCtrl
 */
class ComFrmCtrlSVC_impl
 : public virtual POA_ComFrmCtrl,
   public virtual PortableServer::RefCountServantBase
{
 private:
   // Make sure all instances are built on the heap by making the
   // destructor non-public
   //virtual ~ComFrmCtrlSVC_impl();
   RTC::Logger rtclog;

 public:
  /*!
   * @brief standard constructor
   */
   ComFrmCtrlSVC_impl();
  /*!
   * @brief destructor
   */
   virtual ~ComFrmCtrlSVC_impl();

   // attributes and operations
   void set_param(CORBA::Double k_pv, CORBA::Double v_m, CORBA::Double k_pr);
   int calc_vel(FRAME geo_ref, FRAME geo_cur, VECTOR p_vel, VECTOR r_vel);

  double k_gain, v_max, k_pos_rot;
  double settle;
  int state;

};

/*!
 * @class ComMoveSVC_impl
 * Example class implementing IDL interface ComMove
 */
class ComMoveSVC_impl
 : public virtual POA_ComMove,
   public virtual PortableServer::RefCountServantBase
{
 private:
   // Make sure all instances are built on the heap by making the
   // destructor non-public
   //virtual ~ComMoveSVC_impl();
   RTC::Logger rtclog;

 public:
  /*!
   * @brief standard constructor
   */
   ComMoveSVC_impl();
  /*!
   * @brief destructor
   */
   virtual ~ComMoveSVC_impl();

   int ref_flag;
   FRAME ref_frm;
   // attributes and operations
   void move_to(const Vector& xyz, const Vector& abc);

};



#endif // COM_RMRCSVC_IMPL_H


