// -*- C++ -*-
/*!
 * @file  MatchingComponent.cpp
 * @brief Template Matching Component
 * @date $Date$
 *
 * $Id$
 */

#include "MatchingComponent.h"

// Module specification
// <rtc-template block="module_spec">
static const char* matchingcomponent_spec[] =
  {
    "implementation_id", "MatchingComponent",
    "type_name",         "MatchingComponent",
    "description",       "Template Matching Component",
    "version",           "0.1",
    "vendor",            "OsakaUniv",
    "category",          "Generic",
    "activity_type",     "SPORADIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    "exec_cxt.periodic.rate", "1.0",
    // Configuration variables
    "conf.default.int_width", "640",
    "conf.default.int_height", "480",
    "conf.default.int_match_level", "80",
    "conf.default.int_disp", "0",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
MatchingComponent::MatchingComponent(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_itemposition_servicePort("itemposition_service"),

    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  
  // Set service provider to Ports
  m_itemposition_servicePort.registerProvider("item_position", "item_position", m_item_position);
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  registerPort(m_itemposition_servicePort);
  
  // </rtc-template>

}

/*!
 * @brief destructor
 */
MatchingComponent::~MatchingComponent()
{
}



RTC::ReturnCode_t MatchingComponent::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("int_width", m_int_width, "640");
  bindParameter("int_height", m_int_height, "480");
  bindParameter("int_match_level", m_int_match_level, "80");
  bindParameter("int_disp", m_int_disp, "0");
  
  // </rtc-template>
  return RTC::RTC_OK;
}


RTC::ReturnCode_t MatchingComponent::onFinalize()
{
	printf("onFinalize \n");
	m_item_position.CloseWindow();
	return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t MatchingComponent::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t MatchingComponent::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t MatchingComponent::onActivated(RTC::UniqueId ec_id)
{
  printf("onActivated\n");
  m_item_position.m_int_match_level = m_int_match_level;
  m_item_position.m_int_disp = m_int_disp;
  m_item_position.m_int_height = m_int_height;
  m_item_position.m_int_width = m_int_width;

  if(m_int_disp > 0){   //0:non-display 1:display
    m_item_position.OpenWindow();
  }

  m_item_position.capture = cvCaptureFromCAM( 0);
  if( !m_item_position.capture )
  {
    printf("Could not initialize capturing...\n");
  }

  cvSetCaptureProperty(m_item_position.capture,CV_CAP_PROP_FRAME_WIDTH,m_int_width);
  cvSetCaptureProperty(m_item_position.capture,CV_CAP_PROP_FRAME_HEIGHT,m_int_height);

  m_item_position.cdst = cvLoadImage("photo1.bmp",0);
  m_item_position.cdst_two = cvLoadImage("photo2.bmp",0);
  m_item_position.cdst_three = cvLoadImage("photo3.bmp",0);
  m_item_position.cdst_four = cvLoadImage("photo4.bmp",0);

  return RTC::RTC_OK;
}



RTC::ReturnCode_t MatchingComponent::onDeactivated(RTC::UniqueId ec_id)
{
  printf("onDeactivated \n");
  m_item_position.CloseWindow();

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t MatchingComponent::onExecute(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t MatchingComponent::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t MatchingComponent::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t MatchingComponent::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t MatchingComponent::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t MatchingComponent::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void MatchingComponentInit(RTC::Manager* manager)
  {
    RTC::Properties profile(matchingcomponent_spec);
    manager->registerFactory(profile,
                             RTC::Create<MatchingComponent>,
                             RTC::Delete<MatchingComponent>);
  }
  
};


