// -*- C++ -*-
/*!
 * @file  BackInput.cpp * @brief Swtich input component * $Date$ 
 *
 * $Id$ 
 */
#include "BackInput.h"

// Module specification
// <rtc-template block="module_spec">
static const char* BackInput_spec[] =
  {
    "implementation_id", "BackInput",
    "type_name",         "BackInput",
    "description",       "Swtich input component",
    "version",           "1.0",
    "vendor",            "AIST",
    "category",          "example",
    "activity_type",     "SPORADIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    ""
  };
// </rtc-template>


enum{
  INITIAL_TURN=0,
	LINE_FOLLOW,
	FINAL_TURN,
	STOP
};


BackInput::BackInput(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_outVelOut("velocity", m_outVel),
    m_statusOut("status", m_status),
    m_targetPosIn("targetPos", m_targetPos),
    m_currentPosIn("currentPos", m_currentPos)

    // </rtc-template>
{
}

BackInput::~BackInput()
{
}


RTC::ReturnCode_t BackInput::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
	addInPort("targetPos", m_targetPosIn);
	addInPort("currentPos", m_currentPosIn);

  // Set OutPort buffer
  addOutPort("velocity", m_outVelOut);
  addOutPort("status", m_statusOut);

  // Set service provider to Ports

  // Set service consumers to Ports

  // Set CORBA Service Ports

  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable

  // </rtc-template>

	isGoalSet = 0;
	state = INITIAL_TURN;
	distLim = 0.08;//8cm
	max_w = 0.5;
	max_v = 0.5;
	cycleCnt = 0;
	slope = 12;
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t BackInput::onFinalize()
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t BackInput::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t BackInput::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t BackInput::onActivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t BackInput::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


double angle_diff(double theta1, double theta2){
  double theta_diff;
  theta_diff = theta1 - theta2;

  while(theta_diff < -M_PI)theta_diff+=2*M_PI;
  while(theta_diff > M_PI)theta_diff-=2*M_PI;

  //return fabs(theta_diff);
	return theta_diff;
}

double norm(double x, double y){
	return sqrt(x*x + y*y);
}

double norm_single(double x){
	return sqrt(x*x);
}

double distance(double x, double y, double tx,double ty){
  return norm((x-tx),(y-ty));
}

/*-PI < theta < PIに調整する*/
double trans_q(double theta){
  while(theta > M_PI)theta -= 2.0*M_PI;
  while(theta < -M_PI)theta += 2.0*M_PI;
  return theta;
}

double angle_diff_by_innerProduct(double v1x, double v1y, double v2x, double v2y){
	double theta = acos( (v1x*v2x + v1y*v2y)/norm(v1x,v1y)/norm(v2x,v2y) );
	return trans_q(theta);
}


RTC::ReturnCode_t BackInput::onExecute(RTC::UniqueId ec_id)
{
	//現在位置の読み取り
	if(m_currentPosIn.isNew())
		while(!m_currentPosIn.isEmpty())
			m_currentPosIn.read();

		if(m_targetPosIn.isNew())	
		{
			m_targetPosIn.read();
			RTC_INFO(("direction:%f, goalX:%f, goalY:%f, goalTheta:%f", m_targetPos.data[0], m_targetPos.data[1], m_targetPos.data[2], m_targetPos.data[3]));
			//std::cout << "direction" << m_targetPos.data[0] << ", goalX:" << m_targetPos.data[1] << ", goalY:" << m_targetPos.data[2] << ", goalTheta:" << m_targetPos.data[3] << std::endl;

			bof    = m_targetPos.data[0];
			goalX  = m_targetPos.data[1];
			goalY  = m_targetPos.data[2];
			goalTh = m_targetPos.data[3];
			init   = m_targetPos.data[4];
			end    = m_targetPos.data[5];
			lineLength = distance(goalX, goalY, m_currentPos.x, m_currentPos.y);//予想される直進移動距離
			if(init == 1)
				state = INITIAL_TURN;
			else
				state = LINE_FOLLOW;
			isGoalSet = 1;
		}
		if(isGoalSet)
		{
			dist = distance(goalX, goalY, m_currentPos.x, m_currentPos.y);
			dirAngle = trans_q( M_PI*(bof - 1)/2.0 + atan2( goalY - m_currentPos.y, goalX - m_currentPos.x ) );
			angleDif = angle_diff( dirAngle, m_currentPos.theta );

			switch(state){
			case INITIAL_TURN:
				if( fabs(angleDif) > M_PI/50 )
				{
					m_outVel.vx = 0.0;
					m_outVel.vy = 0.0;
					m_outVel.w  = angleDif;
					if( fabs(m_outVel.w) > max_w)
						m_outVel.w  = max_w * m_outVel.w/norm_single(m_outVel.w);
				}
				else
				{
					m_outVel.vx = 0.0;
					m_outVel.vy = 0.0;
					m_outVel.w  = 0.0;
					state = LINE_FOLLOW;
				}
				m_outVelOut.write();
				break;
			
			case LINE_FOLLOW:
				if( dist > distLim )
				{
					cycleCnt++;
					m_outVel.vx = bof * dist;
					m_outVel.vy = 0.0;
					m_outVel.w  = 2.0 * angleDif;
					if( fabs(m_outVel.w) > max_w)
						m_outVel.w  = 2.0 * max_w * m_outVel.w/norm_single(m_outVel.w);
					if( fabs( m_outVel.vx ) > max_v)
						m_outVel.vx = bof * max_v;
					if(cycleCnt < slope)
					{
						m_outVel.vx = m_outVel.vx * (double)cycleCnt/(double)slope;
						m_outVel.w  = m_outVel.w  * (double)cycleCnt/(double)slope;
					}
				}
				else
				{
					m_outVel.vx = 0.0;
					m_outVel.vy = 0.0;
					m_outVel.w  = 0.0;
					state = FINAL_TURN;
					if(end != 1)
					{
						state = STOP;
						isGoalSet = 0;
						cycleCnt = 0;
					}
				}
				m_outVelOut.write();
				break;
		
			case FINAL_TURN:
				angleDif = angle_diff( goalTh, m_currentPos.theta );
				if( fabs(angleDif) > M_PI/50 )
				{
					m_outVel.vx = 0.0;
					m_outVel.vy = 0.0;
					m_outVel.w  = angleDif;
					if( fabs(m_outVel.w) > max_w)
						m_outVel.w  = max_w * m_outVel.w/norm_single(m_outVel.w);
				}
				else
				{
					m_outVel.vx = 0.0;
					m_outVel.vy = 0.0;
					m_outVel.w  = 0.0;
					state = STOP;
					isGoalSet = 0;
					cycleCnt = 0;
				}
				break;

			default:
				break;

			}
			m_outVelOut.write();
		}

	m_status.data = state;
	m_statusOut.write(m_status);
  usleep(100000);

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t BackInput::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t BackInput::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t BackInput::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t BackInput::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t BackInput::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


extern "C"
{
 
  void BackInputInit(RTC::Manager* manager)
  {
    coil::Properties profile(BackInput_spec);
    manager->registerFactory(profile,
                             RTC::Create<BackInput>,
                             RTC::Delete<BackInput>);
  }
  
};



