// -*- C++ -*-
/*!
 * @file  CeilingNavigation.h
 * @brief CeilingNavigation
 * @date  $Date$
 *
 * $Id$
 */

#ifndef CEILINGNAVIGATION_H
#define CEILINGNAVIGATION_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>

#include "std_hdr.h"

#include "intellirobotStub.h"

#include "BlockMatching.h"
#include "ImageData.h"
#include "CeilingMap.h"

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

using namespace RTC;

/**
 *@brief VirQ[VRTCCNX
 */
class CeilingNavigation
  : public RTC::DataFlowComponentBase
{
 public:
  CeilingNavigation(RTC::Manager* manager);
  ~CeilingNavigation();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  // The initialize action (on CREATED->ALIVE transition)
  // formaer rtc_init_entry() 
   virtual RTC::ReturnCode_t onInitialize();

  // The finalize action (on ALIVE->END transition)
  // formaer rtc_exiting_entry()
  // virtual RTC::ReturnCode_t onFinalize();

  // The startup action when ExecutionContext startup
  // former rtc_starting_entry()
  virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  // The shutdown action when ExecutionContext stop
  // former rtc_stopping_entry()
  virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  // The activated action (Active state entry action)
  // former rtc_active_entry()
  virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  // The deactivated action (Active state exit action)
  // former rtc_active_exit()
  // virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  // The execution action that is invoked periodically
  // former rtc_active_do()
  virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  // The aborting action when main logic error occurred.
  // former rtc_aborting_entry()
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  // The error action in ERROR state
  // former rtc_error_do()
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  // The reset action that is invoked resetting
  // This is same but different the former rtc_init_entry()
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  // The state update action that is invoked after onExecute() action
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  // The action that is invoked when execution context's rate is changed
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // Configuration variable declaration
  // <rtc-template block="config_declare">
	std::string m_NavigationMap;		/* V摜t@C						*/
	unsigned short m_BlockResolution;	/* 摜]\							*/
	unsigned short m_BlockSize;			/* 摜kTCY							*/
	double m_BlockCoefficient;			/* 摜k␳l							*/
	unsigned short m_SearchScope;		/* }b`OT͈						*/
	unsigned short m_BlackWhiteValue;	/* 2l臒l								*/
	long m_Center_X;					/* 摜]SW(X)						*/
	long m_Center_Y;					/* 摜]SW(Y)						*/
	std::string m_OfflineImage;			/* ItCC[Wt@Ci[tH_ */
	double m_RealMapHeight;
	double m_RealMapWidth;
	double m_VirtualMapHeight;
	double m_VirtualMapWidth;

	unsigned long x_pos;
	unsigned long y_pos;
	double theta;

	double mPerPix;//摜1sNZ̎E̒[m]
	//{bgWńCOx,yƂD
	double camX; //{bgWnɂJxW
	double camY; //{bgWnɂJyW
	//pxɂẮC摜̏{bgWxɈv悤ɔzuƂƂŌŒ肷D

  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedOctetSeq m_CameraData;
  InPort<TimedOctetSeq> m_CameraDataIn;
 
//	IIS::TimedPosition m_LocalizedPosition;
//	InPort<IIS::TimedPosition> m_LocalizedPositionIn;
	IIS::TimedPose2D m_LocalizedPosition;
	InPort<IIS::TimedPose2D> m_LocalizedPositionIn; 
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedOdometry m_CeilingOdometry;
  OutPort<TimedOdometry> m_CeilingOdometryOut;

	OutPort<TimedDoubleSeq> m_CeilingOdometrydOut;
	TimedDoubleSeq m_CeilingOdometryd;

  // DataOutPort declaration
  // <rtc-template block="outport_declare">
//  IIS::TimedPosition m_CeilingPosition;
//  OutPort<IIS::TimedPosition> m_CeilingPositionOut;
  IIS::TimedPose2D m_CeilingPosition;
  OutPort<IIS::TimedPose2D> m_CeilingPositionOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:
  int dummy;
  // <rtc-template block="private_attribute">
  
  // </rtc-template>

  // <rtc-template block="private_operation">
  
  // </rtc-template>

  private:
	  BlockMatching	m_BlockMat;
	  CeilingMap	m_CeilingMap;

};


extern "C"
{
  void CeilingNavigationInit(RTC::Manager* manager);
};

#endif // CEILINGNAVIGATION_H
