// -*- C++ -*-
/*!
 * @file  CameraEye.h
 * @brief CameraEye
 * @date  $Date$
 *
 * $Id$
 */

#ifndef CAMERAEYE_H
#define CAMERAEYE_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>

#include <cv.h>
#include <cxcore.h>
#include <highgui.h>


// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

using namespace RTC;

class CameraEye
  : public RTC::DataFlowComponentBase
{
 public:
  CameraEye(RTC::Manager* manager);
  ~CameraEye();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  // The initialize action (on CREATED->ALIVE transition)
  // formaer rtc_init_entry() 
   virtual RTC::ReturnCode_t onInitialize();

  // The finalize action (on ALIVE->END transition)
  // formaer rtc_exiting_entry()
   virtual RTC::ReturnCode_t onFinalize();

  // The startup action when ExecutionContext startup
  // former rtc_starting_entry()
  virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  // The shutdown action when ExecutionContext stop
  // former rtc_stopping_entry()
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  // The activated action (Active state entry action)
  // former rtc_active_entry()
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  // The deactivated action (Active state exit action)
  // former rtc_active_exit()
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  // The execution action that is invoked periodically
  // former rtc_active_do()
	virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  // The aborting action when main logic error occurred.
  // former rtc_aborting_entry()
  virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  // The error action in ERROR state
  // former rtc_error_do()
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  // The reset action that is invoked resetting
  // This is same but different the former rtc_init_entry()
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  // The state update action that is invoked after onExecute() action
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  // The action that is invoked when execution context's rate is changed
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // Configuration variable declaration
  // <rtc-template block="config_declare">


  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedOctetSeq m_ImageData;
  OutPort<TimedOctetSeq> m_ImageDataOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:
  int dummy;
  // <rtc-template block="private_attribute">
  
  // </rtc-template>

  // <rtc-template block="private_operation">
  
  // </rtc-template>
	CvCapture	*cap;
	IplImage	*img;
 	IplImage *hikaku_img;//位置合わせ用画像
	int			m_Flip;
	int			m_CapSerial;
	std::string	m_caribFile;
	std::string m_captureWindow;		//画像のウィンドウ表示(on)/非表示(off)
	char captureWindow[4];
	int m_usbID; // USBカメラのID /dev/video# の#の番号
	int window_flag; //ウィンドウ表示(0), 非表示(1)
	int cap_window;

	CvSeq* m_image_points_seq;
	CvMat* m_image_points_mat;
	CvSize m_img_size;
	CvSize m_board_size;
	float m_square_size;
	float m_aspect_ratio;
	int m_flags;
	CvMat* m_camera_matrix;
	CvMat* m_dist_coeffs;
	CvMat* m_extr_params;
	CvMat* m_reproj_errs;
	double m_avg_reproj_err ;
	double m_magnification;
	CvRect roi;

	double compute_reprojection_error( const CvMat* object_points,
		    const CvMat* rot_vects, const CvMat* trans_vects,
		    const CvMat* camera_matrix, const CvMat* dist_coeffs,
		    const CvMat* image_points, const CvMat* point_counts,
		    CvMat* per_view_errors );
	int load_calibration(std::string caribFile);
	int run_calibration();

};


extern "C"
{
  void CameraEyeInit(RTC::Manager* manager);
};


#endif // CAMERAEYE_H
