#!/usr/bin/env python
# -*- Python -*-

import sys
import time
import os

#
# set up user environment
#  RtmToolsDir, RtmDir, etc.
#
sys.path.append("..")
from set_env_sim import *   

#
# import user tools
#
sys.path.append(".")
save_path = sys.path[:]
sys.path.append(RtmToolsDir+'/rtc_handle')
from rtc_handle import *
sys.path = save_path

#
# import stub files
# 
#import _GlobalIDL

#
# run components
#
def run_components() :
  com_rh="exec gnome-terminal "
  com_rh+=" --tab-with-profile=default --working-directory="+Rh2BinDir+"/comp -t Odometry -e \"./OdometryComp -f "+Rh2ConfDir+"/Odometry.conf\""
  com_rh+=" --tab-with-profile=default --working-directory="+Rh2BinDir+"/script/openhrp3 -t bridge -e \"sh bridge_withoutSensor.sh\""
  com_rh+=" --tab-with-profile=default --working-directory="+Rh2BinDir+"/comp -t InitOdometry -e \"./InitOdometryComp -f "+Rh2ConfDir+"/InitOdometry.conf\""
  os.popen2(com_rh)

  com_sj="exec gnome-terminal "
  com_sj+=" --tab-with-profile=default --working-directory="+Rh2BinDir+"/comp -t Navigation -e \"./NavigationComp -f "+Rh2ConfDir+"/Navigation.conf\""
  com_sj+=" --tab-with-profile=default --working-directory="+Rh2BinDir+"/comp -t PathFollower -e \" ./PathFollowerComp -f "+Rh2ConfDir+"/PathFollower.conf\""
  com_sj+=" --tab-with-profile=default --working-directory="+Rh2BinDir+"/comp -t Dijkstra -e \" ./DijkstraComp -f "+Rh2ConfDir+"/Dijkstra.conf\""
  com_sj+=" --tab-with-profile=default --working-directory="+Rh2BinDir+"/comp -t TestInputSTART -e \"./TestInputComp -f "+Rh2ConfDir+"/Start.conf\""
  com_sj+=" --tab-with-profile=default --working-directory="+Rh2BinDir+"/comp -t TestInputGOAL -e \"./TestInputComp -f "+Rh2ConfDir+"/Goal.conf\""
  com_sj+=" --tab-with-profile=default --working-directory="+Rh2BinDir+"/comp -t HazardMesh -e \" ./HazardMeshComp -f "+Rh2ConfDir+"/HazardMesh.conf\""
  com_sj+=" --tab-with-profile=default --working-directory="+Rh2BinDir+"/comp -t ManagementOfAreaMap -e \" ./ManagementOfAreaMapComp -f "+Rh2ConfDir+"/ManagementOfAreaMap.conf\""
  com_sj+=" --tab-with-profile=default --working-directory="+Rh2BinDir+"/comp -t PathCnv -e \" ./PathCnvComp -f "+Rh2ConfDir+"/PathCnv.conf\""
  os.popen2(com_sj)

def assign_variables(env) :
  # rtc
  global odometry,initodometry
  global navigation, pathfollower
  global motor,dijkstra, hazardmesh, managementofareamap, pathcnv
  global teststart, testgoal
  #global localization, switch
  odometry = env.name_space[NS0].rtc_handles['Odometry0.rtc']
  initodometry = env.name_space[NS0].rtc_handles['InitOdometry0.rtc']
  motor = env.name_space[NS0].rtc_handles['MotorControl0.rtc']
  navigation = env.name_space[NS0].rtc_handles['Navigation0.rtc']
  pathfollower = env.name_space[NS0].rtc_handles['PathFollower0.rtc']
  teststart =env.name_space[NS0].rtc_handles['TestInput0START.rtc']
  testgoal = env.name_space[NS0].rtc_handles['TestInput0GOAL.rtc']
  dijkstra = env.name_space[NS0].rtc_handles['Dijkstra0.rtc']
  hazardmesh = env.name_space[NS0].rtc_handles['HazardMesh0.rtc']
  managementofareamap = env.name_space[NS0].rtc_handles['ManagementOfAreaMap0.rtc']
  pathcnv = env.name_space[NS0].rtc_handles['PathCnv0.rtc']
  #localization = env.name_space[NS0].rtc_handles['LocalizeCenter0.rtc']
  #switch = env.name_space[NS0].rtc_handles['SwitchInputRTC0.rtc']

def make_connectors(env):
  env.con_list=[]

  ## [ManagementOfAreaMap] -> [HazardMesh]
  con = IOConnector([managementofareamap.outports['MapData'],hazardmesh.inports['MapData']])
  env.con_list.append(con)
  ## [HazardMesh] -> [ManagementOfAreaMap]
  con = IOConnector([hazardmesh.outports['MapOrder'],managementofareamap.inports['MapOrder']])
  env.con_list.append(con)
  ## [HazardMesh] -> [Dijkstra]
  con = IOConnector([hazardmesh.outports['MeshData'],dijkstra.inports['MeshData']])
  env.con_list.append(con)
  ## [TestInput] -> [Dijkstra]
  con = IOConnector([teststart.outports['Position'],dijkstra.inports['RobotPosition']])
  env.con_list.append(con)
  con = IOConnector([testgoal.outports['Position'],dijkstra.inports['GoalPosition']])
  env.con_list.append(con)
  ## [Dijkstra] -> [HazardMesh]
  con = IOConnector([dijkstra.outports['MeshOrder'],hazardmesh.inports['MeshOrder']])
  env.con_list.append(con)
  ## [Dijkstra] -> [PathCnv]
  con = IOConnector([dijkstra.outports['Path'],pathcnv.inports['Path']])
  env.con_list.append(con)
  ## [PathCnv] -> [Navigation]
  con = IOConnector([pathcnv.outports['min_path'],navigation.inports['min_path']])
  env.con_list.append(con)
  ## [Navigation] -> [PathFollower]
  con = IOConnector([navigation.outports['path'],pathfollower.inports['target_path']])
  env.con_list.append(con)
  con = IOConnector([navigation.outports['max_speed'],pathfollower.inports['max_speed']])
  env.con_list.append(con)
  con = IOConnector([navigation.outports['max_acceleration'],pathfollower.inports['max_acceleration']])
  env.con_list.append(con)
  ## [PathFollower] -> [MotorControl]
  con = IOConnector([pathfollower.outports['velocity'],motor.inports['TargetVelocity']])
  env.con_list.append(con)
  ## [MotorControl] -> [Odometry]
  con = IOConnector([motor.outports['OutCurrentWheelAngle'],odometry.inports['CurrentWheelAngle']])
  env.con_list.append(con)
  ## [Odometry] -> [Navigation]
  con = IOConnector([odometry.outports['OdometryPosition'],navigation.inports['position']])
  env.con_list.append(con)
  ## [Odometry] -> [PathFollower]
  con = IOConnector([odometry.outports['OdometryPosition'],pathfollower.inports['position']])
  env.con_list.append(con)
  ## [InitOdometry] -> [Odometry]
  con = IOConnector([initodometry.outports['Position'],odometry.inports['LocalizedPosition']])
  env.con_list.append(con)

def set_config_param() :
  teststart.conf_ref.activate_configuration_set('start')
  testgoal.conf_ref.activate_configuration_set('entrance')
  initodometry.conf_ref.activate_configuration_set('start')
  dijkstra.conf_ref.activate_configuration_set('rtc-center')
  navigation.conf_ref.activate_configuration_set('rtc-center')

#
# connection control
#
def connect_all(env) :
  for con in env.con_list :
    con.connect()
def disconnect_all(env) :
  for con in env.con_list :
    con.disconnect()
def connect_list(env,ll) :
  for i in ll :
    env.con_list[i].connect()
def disconnect_list(env,ll) :
  for i in ll :
    env.con_list[i].disconnect()
#
#
#

def activate_all() :
  odometry.activate()
  pathfollower.activate()
  navigation.activate()
  pathcnv.activate()
  dijkstra.activate()
  hazardmesh.activate()
  managementofareamap.activate()
  initodometry.activate()
  teststart.activate()
  testgoal.activate()
def deactivate_all() :
  odometry.deactivate()
  pathfollower.deactivate()
  navigation.deactivate()
  pathcnv.deactivate()
  dijkstra.deactivate()
  hazardmesh.deactivate()
  managementofareamap.deactivate()
  initodometry.deactivate()
  teststart.deactivate()
  testgoal.deactivate()
#
#
def moveto(start, goal):
  pos=['start','entrance','room','drinkserve','drinkserve_back','dumpsite']
  s_flg=0
  g_flg=0
  for i in range(len(pos)):
    if(start == pos[i]):
      s_pos=pos[i]
      print "start position:" + s_pos
      s_flg=1
    if(goal == pos[i]):
      g_pos=pos[i]
      print "start position:" + g_pos
      g_flg=1
  if(s_flg == 1 and g_flg == 1):
      teststart.conf_ref.activate_configuration_set(s_pos)
      testgoal.conf_ref.activate_configuration_set(g_pos)
      pathfollower.activate()
      navigation.activate()
      dijkstra.activate()
      hazardmesh.activate()
      managementofareamap.activate()
      pathcnv.activate()
      teststart.activate()
      testgoal.activate()
  else:
      print "error:start or goal is invalid."

def stop():
  pathcnv.deactivate()
  navigation.deactivate()
  pathfollower.deactivate()
  dijkstra.deactivate()
  hazardmesh.deactivate()
  managementofareamap.deactivate()
  teststart.deactivate()
  testgoal.deactivate()

#
# create env
#
env = RtmEnv(sys.argv,[NS0])
run_components()
time.sleep(10)
raw_input("are you sure that each rtc needed is running?")
for ns in env.name_space :
  env.name_space[ns].list_obj()
assign_variables(env)
make_connectors(env)
connect_all(env)
set_config_param()
odometry.activate()
time.sleep(1)
initodometry.activate()
